/* file sttrend.c	G. Moody        13 May 1999
			Last revised:  3 October 1999

-------------------------------------------------------------------------------
sttrend: Generate ST and instantaneous heart rate trends from annotation file
Copyright (C) 1999 George B. Moody

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (george@mit.edu) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit the author's web site (http://ecg.mit.edu/).
_______________________________________________________________________________

This program is based on ihr.c, from the WFDB Software Package.
*/

#include <stdio.h>
#ifndef __STDC__
extern void exit();
#endif

#include <wfdb/wfdb.h>
#define map2
#define ammap
#define mamap
#define annpos
#include <wfdb/ecgmap.h>
#define MAXREFS 1024     /* maximum number of local references per channel */

char *pname;
static int rst0[MAXREFS], rst1[MAXREFS], nr0, nr1;
static long rt0[MAXREFS], rt1[MAXREFS];

int initref(char *s)
{
  FILE *ifile;
  int c, rst;
  long t;
  char buf[128];

  if ((ifile = fopen(s, "rt")) == NULL) return (0);
  while (fgets(buf, sizeof(buf), ifile)) {
    sscanf(buf, "%ld %d %d", &t, &c, &rst);
    if (c == 0 && t > rt0[nr0] && ++nr0 < MAXREFS) {
      rst0[nr0] = rst;
      rt0[nr0] = t;
    }
    else if (c == 1 && t > rt1[nr1] && ++nr1 < MAXREFS) {
      rst1[nr1] = rst;
      rt1[nr1] = t;
    }
  }
  rst0[0] = rst0[1];
  rst1[0] = rst1[1];
  rst0[nr0+1] = rst0[nr0];
  rst1[nr1+1] = rst1[nr1];
  rt0[nr0+1] = rt1[nr1+1] = strtim("25:0:0");
  return (1);
}

double localref(int s, long t)
{
  static int sti0, sti1;
  double lref = 0.0;

  if (s == 0) {
    while (t >= rt0[sti0+1])
      sti0++;
    lref = rst0[sti0] + (t - rt0[sti0]) * (rst0[sti0+1] - rst0[sti0]) /
                                           (rt0[sti0+1] - rt0[sti0]);
  }
  else if (s == 1) { 
    while (t >= rt1[sti1+1])
      sti1++;
    lref = rst1[sti1] + (t - rt1[sti1]) * (rst1[sti1+1] - rst1[sti1]) /
                                           (rt1[sti1+1] - rt1[sti1]);
  }
  return (lref);
}

main(argc, argv)
int argc;
char *argv[];
{
    char *record = NULL, *prog_name();
    double ihr, ihrlast, mhr = 70., sph, spm, sps, tol = 10.0, atof(), fabs();
    int cflag = 0, i, j;
    double st0, st1;
    long from = 0L, to = 0L, lasttime = 0L;
    static WFDB_Anninfo ai;
    WFDB_Annotation annot;
    void help();

    pname = prog_name(argv[0]);

    /* Interpret command-line options. */
    for (i = 1; i < argc; i++) {
	if (*argv[i] == '-') switch (*(argv[i]+1)) {
	  case 'a':	/* annotator follows */
	    if (++i >= argc) {
		(void)fprintf(stderr, "%s: annotator must follow -a\n",
			      pname);
		exit(1);
	    }
	    ai.name = argv[i];
	    break;
	  case 'c':	/* correct trends (subtract local references) */
	    cflag = 1;
	    break;
	  case 'd':	/* tolerance (max HR change beat to beat, in bpm) */
	    if (++i >= argc) {
		(void)fprintf(stderr, "%s: tolerance must follow -d\n",
			      pname);
		exit(1);
	    }
	    tol = atof(argv[i]);
	    break;
	  case 'f':	/* starting time follows */
	    if (++i >= argc) {
		(void)fprintf(stderr, "%s: starting time must follow -f\n",
			      pname);
		exit(1);
	    }
	    from = i;		/* to be converted to sample intervals below */
	    break;
	  case 'h':	/* print usage summary and quit */
	    help();
	    exit(0);
	    break;
	  case 'r':	/* input record name follows */
	    if (++i >= argc) {
		(void)fprintf(stderr,
			      "%s: input record name must follow -r\n",
			      pname);
		exit(1);
	    }
	    record = argv[i];
	    break;
	  case 't':	/* ending time follows */
	    if (++i >= argc) {
		(void)fprintf(stderr, "%s: end time must follow -t\n", pname);
		exit(1);
	    }
	    to = i;
	    break;
	  default:
	    (void)fprintf(stderr, "%s: unrecognized option %s\n",
			  pname, argv[i]);
	    exit(1);
	}
	else {
	    (void)fprintf(stderr, "%s: unrecognized argument %s\n",
			  pname, argv[i]);
	    exit(1);
	}
    }
    if (record == NULL || ai.name == NULL) {
	help();
	exit(1);
    }

    if ((sps = sampfreq(record)) < 0.)
	(void)setsampfreq(sps = WFDB_DEFFREQ);
    spm = sps * 60.0;
    sph = spm * 60.0;

    if (cflag) {
      char strefname[20];

      sprintf(strefname, "stref.%s", record);
      if (initref(strefname) == 0) {
	fprintf(stderr, "can't open '%s' -- run stref and try again\n",
		strefname);
	exit(2);
      }
    }
    ai.stat = WFDB_READ;
    if (annopen(record, &ai, 1) < 0) /* open annotation file */
	exit(2);

    if (from && iannsettime(strtim(argv[(int)from])) < 0) exit(2);
    if (to) to = strtim(argv[(int)to]);

    while (getann(0, &annot) == 0 && (to == 0L || annot.time <= to)) {
	char *p, *q;

	if (annot.anntyp != STCH) continue;
	if (annot.chan == 1) {
	    p = annot.aux+4;
	    while (*p != '+' && *p != '-')
		p++;
	    q = p+1;
	    st1 = 0.0;
	    while ('0' <= *q && *q <= '9') {
		st1 *= 10.0;
		st1 += (double)(*q++ - '0');
	    }
	    if (*p == '-') st1 = -st1;
	    if (cflag) {
	      st0 -= localref(0, lasttime);
	      st1 -= localref(1, lasttime);
	    }
	    ihr = sps*60./(annot.time - lasttime);
	    mhr += (ihr - mhr)/10.;
	    if (fabs(ihr - ihrlast) < tol && fabs(ihr - mhr) < tol)
		(void)printf("%g %g %g %g\n", lasttime/sph, ihr, st0, st1);
	    lasttime = annot.time;
	    ihrlast = ihr;
	}
	else {
	    p = annot.aux+4;
	    while (*p != '+' && *p != '-')
		p++;
	    q = p+1;
	    st0 = 0.0;
	    while ('0' <= *q && *q <= '9') {
		st0 *= 10.0;
		st0 += (double)(*q++ - '0');
	    }
	    if (*p == '-') st0 = -st0;
	}
    }
    exit(0);			/*NOTREACHED*/
}

char *prog_name(s)
char *s;
{
    char *p = s + strlen(s);

#ifdef MSDOS
    while (p >= s && *p != '\\' && *p != ':') {
	if (*p == '.')
	    *p = '\0';			/* strip off extension */
	if ('A' <= *p && *p <= 'Z')
	    *p += 'a' - 'A';		/* convert to lower case */
	p--;
    }
#else
    while (p >= s && *p != '/')
	p--;
#endif
    return (p+1);
}

static char *help_strings[] = {
    "usage: %s -r RECORD -a ANNOTATOR [OPTIONS ...]\n",
    "where RECORD and ANNOTATOR specify the input, and OPTIONS may include:",
    " -d TOL   reject beat-to-beat HR changes > TOL bpm",
    " -f TIME  start at specified TIME",
    " -h       print this usage summary",
    " -t TIME  stop at specified TIME",
    "Output is four columns: elapsed time (hours), HR (bpm), ST0, ST1",
    NULL
};

void help()
{
    int i;

    (void)fprintf(stderr, help_strings[0], pname);
    for (i = 1; help_strings[i] != NULL; i++)
	(void)fprintf(stderr, "%s\n", help_strings[i]);
}
